@@ -13,6 +13,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+#include <cmath>
 #include <unordered_map>
 #include <regex>
 
@@ -33,6 +34,17 @@ std::string Argument::ctype() const {
   return (s < 0) ? tfm::format("int%d_t", -s) : tfm::format("uint%d_t", s);
 }
 
+static const char *type_names[][4] = {
+  { "int8_t", "int16_t", "int32_t", "int64_t" },
+  { "uint8_t", "uint16_t", "uint32_t", "uint64_t" },
+};
+
+const char *Argument::ctype_name() const {
+  const int s = arg_size();
+  const int r = log2(abs(s));
+  return s < 0 ? type_names[0][r] : type_names[1][r];
+}
+
 bool Argument::get_global_address(uint64_t *address, const std::string &binpath,
                                   const optional<int> &pid) const {
   if (pid) {
@@ -64,12 +76,18 @@ bool Argument::assign_to_local(std::ostream &stream,
                                const std::string &binpath,
                                const optional<int> &pid) const {
   if (constant_) {
-    tfm::format(stream, "%s = %d;", local_name, *constant_);
+    tfm::format(stream, "%s = %lld;", local_name, *constant_);
     return true;
   }
 
   if (!deref_offset_) {
-    tfm::format(stream, "%s = ctx->%s;", local_name, *base_register_name_);
+    if(base_register_name_->substr(0,3) == "xmm") {
+      // TODO: When we can read xmm registers from BPF, update this to read
+      // the actual value
+      tfm::format(stream, "%s = 0;", local_name);
+    } else {
+      tfm::format(stream, "%s = ctx->%s;", local_name, *base_register_name_);
+    }
     // Put a compiler barrier to prevent optimization
     // like llvm SimplifyCFG SinkThenElseCodeToEnd
     // Volatile marking is not sufficient to prevent such optimization.
@@ -172,16 +190,27 @@ bool ArgumentParser_aarch64::parse_size(ssize_t pos, ssize_t &new_pos,
 }
 
 bool ArgumentParser_aarch64::parse_mem(ssize_t pos, ssize_t &new_pos,
-                                       std::string &reg_name,
-                                       optional<int> *offset) {
-  if (parse_register(pos, new_pos, reg_name) == false)
+                                       Argument *dest) {
+  std::string base_reg_name, index_reg_name;
+
+  if (parse_register(pos, new_pos, base_reg_name) == false)
     return false;
+  dest->base_register_name_ = base_reg_name;
 
   if (arg_[new_pos] == ',') {
     pos = new_pos + 1;
-    new_pos = parse_number(pos, offset);
-    if (new_pos == pos)
-      return error_return(pos, pos);
+    new_pos = parse_number(pos, &dest->deref_offset_);
+    if (new_pos == pos) {
+      // offset isn't a number, so it should be a reg,
+      // which looks like: -1@[x0, x1], rather than -1@[x0, 24]
+      skip_whitespace_from(pos);
+      pos = cur_pos_;
+      if (parse_register(pos, new_pos, index_reg_name) == false)
+        return error_return(pos, pos);
+      dest->index_register_name_ = index_reg_name;
+      dest->scale_ = 1;
+      dest->deref_offset_ = 0;
+    }
   }
   if (arg_[new_pos] != ']')
     return error_return(new_pos, new_pos);
@@ -196,6 +225,7 @@ bool ArgumentParser_aarch64::parse(Argument *dest) {
   // Support the following argument patterns:
   //   [-]<size>@<value>, [-]<size>@<reg>, [-]<size>@[<reg>], or
   //   [-]<size>@[<reg>,<offset>]
+  //   [-]<size>@[<reg>,<index_reg>]
   ssize_t cur_pos = cur_pos_, new_pos;
   optional<int> arg_size;
 
@@ -218,17 +248,128 @@ bool ArgumentParser_aarch64::parse(Argument *dest) {
     cur_pos_ = new_pos;
     dest->base_register_name_ = reg_name;
   } else if (arg_[cur_pos] == '[') {
-    // Parse ...@[<reg>] and ...@[<reg,<offset>]
-    optional<int> offset = 0;
-    std::string reg_name;
-    if (parse_mem(cur_pos + 1, new_pos, reg_name, &offset) == false)
+    // Parse ...@[<reg>], ...@[<reg,<offset>] and ...@[<reg>,<index_reg>]
+    if (parse_mem(cur_pos + 1, new_pos, dest) == false)
       return false;
     cur_pos_ = new_pos;
-    dest->base_register_name_ = reg_name;
-    dest->deref_offset_ = offset;
   } else {
     // Parse ...@<value>
-    optional<int> val;
+    optional<long long> val;
+    new_pos = parse_number(cur_pos, &val);
+    if (cur_pos == new_pos)
+      return error_return(cur_pos, cur_pos);
+    cur_pos_ = new_pos;
+    dest->constant_ = val;
+  }
+
+  skip_whitespace_from(cur_pos_);
+  return true;
+}
+
+bool ArgumentParser_loongarch64::parse_register(ssize_t pos, ssize_t &new_pos,
+						std::string &reg_name) {
+  if (arg_[pos] == '$' && arg_[pos + 1] == 'r') {
+    optional<int> reg_num;
+    new_pos = parse_number(pos + 2, &reg_num);
+    if (new_pos == pos + 2 || *reg_num < 0 || *reg_num > 31)
+      return error_return(pos + 2, pos + 2);
+
+    if (*reg_num == 3) {
+      reg_name = "sp";
+    } else {
+      reg_name = "regs[" + std::to_string(reg_num.value()) + "]";
+    }
+    return true;
+  } else if (arg_[pos] == 's' && arg_[pos + 1] == 'p') {
+    reg_name = "sp";
+    new_pos = pos + 2;
+    return true;
+  } else {
+    return error_return(pos, pos);
+  }
+}
+
+bool ArgumentParser_loongarch64::parse_size(ssize_t pos, ssize_t &new_pos,
+					    optional<int> *arg_size) {
+  int abs_arg_size;
+
+  new_pos = parse_number(pos, arg_size);
+  if (new_pos == pos)
+    return error_return(pos, pos);
+
+  abs_arg_size = abs(arg_size->value());
+  if (abs_arg_size != 1 && abs_arg_size != 2 && abs_arg_size != 4 &&
+      abs_arg_size != 8)
+    return error_return(pos, pos);
+  return true;
+}
+
+bool ArgumentParser_loongarch64::parse_mem(ssize_t pos, ssize_t &new_pos,
+					   Argument *dest) {
+  std::string base_reg_name, index_reg_name;
+
+  if (parse_register(pos, new_pos, base_reg_name) == false)
+    return false;
+  dest->base_register_name_ = base_reg_name;
+
+  if (arg_[new_pos] == ',') {
+    pos = new_pos + 1;
+    new_pos = parse_number(pos, &dest->deref_offset_);
+    if (new_pos == pos) {
+      // offset isn't a number, so it should be a reg,
+      // which looks like: -1@[$r0, $r1], rather than -1@[$r0, 24]
+      skip_whitespace_from(pos);
+      pos = cur_pos_;
+      if (parse_register(pos, new_pos, index_reg_name) == false)
+        return error_return(pos, pos);
+      dest->index_register_name_ = index_reg_name;
+      dest->scale_ = 1;
+      dest->deref_offset_ = 0;
+    }
+  }
+  if (arg_[new_pos] != ']')
+    return error_return(new_pos, new_pos);
+  new_pos++;
+  return true;
+}
+
+bool ArgumentParser_loongarch64::parse(Argument *dest) {
+  if (done())
+    return false;
+
+  // Support the following argument patterns:
+  //   [-]<size>@<value>, [-]<size>@<reg>, [-]<size>@[<reg>], or
+  //   [-]<size>@[<reg>,<offset>]
+  //   [-]<size>@[<reg>,<index_reg>]
+  ssize_t cur_pos = cur_pos_, new_pos;
+  optional<int> arg_size;
+
+  // Parse [-]<size>
+  if (parse_size(cur_pos, new_pos, &arg_size) == false)
+    return false;
+  dest->arg_size_ = arg_size;
+
+  // Make sure '@' present
+  if (arg_[new_pos] != '@')
+    return error_return(new_pos, new_pos);
+  cur_pos = new_pos + 1;
+
+  if (arg_[cur_pos] == '$' || arg_[cur_pos] == 's') {
+    // Parse ...@<reg>
+    std::string reg_name;
+    if (parse_register(cur_pos, new_pos, reg_name) == false)
+      return false;
+
+    cur_pos_ = new_pos;
+    dest->base_register_name_ = reg_name;
+  } else if (arg_[cur_pos] == '[') {
+    // Parse ...@[<reg>], ...@[<reg,<offset>] and ...@[<reg>,<index_reg>]
+    if (parse_mem(cur_pos + 1, new_pos, dest) == false)
+      return false;
+    cur_pos_ = new_pos;
+  } else {
+    // Parse ...@<value>
+    optional<long long> val;
     new_pos = parse_number(cur_pos, &val);
     if (cur_pos == new_pos)
       return error_return(cur_pos, cur_pos);
@@ -268,7 +409,7 @@ bool ArgumentParser_powerpc64::parse(Argument *dest) {
     arg_str = &arg_[cur_pos_];
 
     if (std::regex_search(arg_str, matches, arg_op_regex_const)) {
-      dest->constant_ = stoi(matches.str(1));
+      dest->constant_ = (long long)stoull(matches.str(1));
     } else if (std::regex_search(arg_str, matches, arg_op_regex_reg)) {
       dest->base_register_name_ = "gpr[" + matches.str(1) + "]";
     } else if (std::regex_search(arg_str, matches, arg_op_regex_breg_off)) {
@@ -327,7 +468,7 @@ bool ArgumentParser_s390x::parse(Argument *dest) {
     cur_pos_ += matches.length(0);
 
     if (std::regex_search(arg_ + cur_pos_, matches, arg_op_regex_imm)) {
-      dest->constant_ = stoi(matches.str(1));
+      dest->constant_ = (long long)stoull(matches.str(1));
     } else if (std::regex_search(arg_ + cur_pos_, matches, arg_op_regex_reg)) {
       dest->base_register_name_ = "gprs[" + matches.str(1) + "]";
     } else if (std::regex_search(arg_ + cur_pos_, matches, arg_op_regex_mem)) {
@@ -355,6 +496,56 @@ bool ArgumentParser_s390x::parse(Argument *dest) {
   return true;
 }
 
+
+
+bool ArgumentParser_riscv64::parse(Argument *dest) {
+  if (done())
+    return false;
+
+  bool matched;
+  std::smatch matches;
+  std::string arg_str(&arg_[cur_pos_]);
+  std::regex arg_n_regex("^(\\-?[1248])\\@");
+  // Operands with constants of form NUM
+  std::regex arg_op_regex_const("^(\\-?[0-9]+)( +|$)");
+  std::string reg_str =
+        "((a[0-9])|((s[0-9p])|s1[0-1])|(t[0-6p])|(pc)|(ra)|(fp)|(gp))";
+  // Operands with register only of form REG
+  std::regex arg_op_regex_reg("^" + reg_str + "( +|$)");
+  // Operands with a base register and an offset of form NUM(REG) or -NUM(REG)
+  std::regex arg_op_regex_breg_off("^(\\-?[0-9]+)\\(" + reg_str + "\\)( +|$)");
+
+  matched = std::regex_search(arg_str, matches, arg_n_regex);
+  if (matched) {
+    dest->arg_size_ = stoi(matches.str(1));
+    cur_pos_ += matches.length(0);
+    arg_str = &arg_[cur_pos_];
+    std::string reg_name;
+
+    if (std::regex_search(arg_str, matches, arg_op_regex_const)) {
+      dest->constant_ = (long long)stoull(matches.str(1));
+    } else if (std::regex_search(arg_str, matches, arg_op_regex_reg)) {
+      dest->base_register_name_ = matches.str(1);
+    } else if (std::regex_search(arg_str, matches, arg_op_regex_breg_off)) {
+      dest->deref_offset_ = stoi(matches.str(1));
+      dest->base_register_name_ = matches.str(2);
+    } else {
+        matched = false;
+    }
+  }
+
+  if (!matched) {
+    print_error(cur_pos_);
+    skip_until_whitespace_from(cur_pos_);
+    skip_whitespace_from(cur_pos_);
+    return false;
+  }
+
+  cur_pos_ += matches.length(0);
+  skip_whitespace_from(cur_pos_);
+  return true;
+}
+
 ssize_t ArgumentParser_x64::parse_identifier(ssize_t pos,
                                              optional<std::string> *result) {
   if (isalpha(arg_[pos]) || arg_[pos] == '_') {
@@ -483,113 +674,180 @@ bool ArgumentParser_x64::parse(Argument *dest) {
 
 const std::unordered_map<std::string, ArgumentParser_x64::RegInfo>
     ArgumentParser_x64::registers_ = {
-        {"rax", {REG_A, 8}},   {"eax", {REG_A, 4}},
-        {"ax", {REG_A, 2}},    {"al", {REG_A, 1}},
+        {"rax", {X64_REG_A, 8}},   {"eax", {X64_REG_A, 4}},
+        {"ax", {X64_REG_A, 2}},    {"al", {X64_REG_A, 1}},
 
-        {"rbx", {REG_B, 8}},   {"ebx", {REG_B, 4}},
-        {"bx", {REG_B, 2}},    {"bl", {REG_B, 1}},
+        {"rbx", {X64_REG_B, 8}},   {"ebx", {X64_REG_B, 4}},
+        {"bx", {X64_REG_B, 2}},    {"bl", {X64_REG_B, 1}},
 
-        {"rcx", {REG_C, 8}},   {"ecx", {REG_C, 4}},
-        {"cx", {REG_C, 2}},    {"cl", {REG_C, 1}},
+        {"rcx", {X64_REG_C, 8}},   {"ecx", {X64_REG_C, 4}},
+        {"cx", {X64_REG_C, 2}},    {"cl", {X64_REG_C, 1}},
 
-        {"rdx", {REG_D, 8}},   {"edx", {REG_D, 4}},
-        {"dx", {REG_D, 2}},    {"dl", {REG_D, 1}},
+        {"rdx", {X64_REG_D, 8}},   {"edx", {X64_REG_D, 4}},
+        {"dx", {X64_REG_D, 2}},    {"dl", {X64_REG_D, 1}},
 
-        {"rsi", {REG_SI, 8}},  {"esi", {REG_SI, 4}},
-        {"si", {REG_SI, 2}},   {"sil", {REG_SI, 1}},
+        {"rsi", {X64_REG_SI, 8}},  {"esi", {X64_REG_SI, 4}},
+        {"si", {X64_REG_SI, 2}},   {"sil", {X64_REG_SI, 1}},
 
-        {"rdi", {REG_DI, 8}},  {"edi", {REG_DI, 4}},
-        {"di", {REG_DI, 2}},   {"dil", {REG_DI, 1}},
+        {"rdi", {X64_REG_DI, 8}},  {"edi", {X64_REG_DI, 4}},
+        {"di", {X64_REG_DI, 2}},   {"dil", {X64_REG_DI, 1}},
 
-        {"rbp", {REG_BP, 8}},  {"ebp", {REG_BP, 4}},
-        {"bp", {REG_BP, 2}},   {"bpl", {REG_BP, 1}},
+        {"rbp", {X64_REG_BP, 8}},  {"ebp", {X64_REG_BP, 4}},
+        {"bp", {X64_REG_BP, 2}},   {"bpl", {X64_REG_BP, 1}},
 
-        {"rsp", {REG_SP, 8}},  {"esp", {REG_SP, 4}},
-        {"sp", {REG_SP, 2}},   {"spl", {REG_SP, 1}},
+        {"rsp", {X64_REG_SP, 8}},  {"esp", {X64_REG_SP, 4}},
+        {"sp", {X64_REG_SP, 2}},   {"spl", {X64_REG_SP, 1}},
 
-        {"r8", {REG_8, 8}},    {"r8d", {REG_8, 4}},
-        {"r8w", {REG_8, 2}},   {"r8b", {REG_8, 1}},
+        {"r8", {X64_REG_8, 8}},    {"r8d", {X64_REG_8, 4}},
+        {"r8w", {X64_REG_8, 2}},   {"r8b", {X64_REG_8, 1}},
 
-        {"r9", {REG_9, 8}},    {"r9d", {REG_9, 4}},
-        {"r9w", {REG_9, 2}},   {"r9b", {REG_9, 1}},
+        {"r9", {X64_REG_9, 8}},    {"r9d", {X64_REG_9, 4}},
+        {"r9w", {X64_REG_9, 2}},   {"r9b", {X64_REG_9, 1}},
 
-        {"r10", {REG_10, 8}},  {"r10d", {REG_10, 4}},
-        {"r10w", {REG_10, 2}}, {"r10b", {REG_10, 1}},
+        {"r10", {X64_REG_10, 8}},  {"r10d", {X64_REG_10, 4}},
+        {"r10w", {X64_REG_10, 2}}, {"r10b", {X64_REG_10, 1}},
 
-        {"r11", {REG_11, 8}},  {"r11d", {REG_11, 4}},
-        {"r11w", {REG_11, 2}}, {"r11b", {REG_11, 1}},
+        {"r11", {X64_REG_11, 8}},  {"r11d", {X64_REG_11, 4}},
+        {"r11w", {X64_REG_11, 2}}, {"r11b", {X64_REG_11, 1}},
 
-        {"r12", {REG_12, 8}},  {"r12d", {REG_12, 4}},
-        {"r12w", {REG_12, 2}}, {"r12b", {REG_12, 1}},
+        {"r12", {X64_REG_12, 8}},  {"r12d", {X64_REG_12, 4}},
+        {"r12w", {X64_REG_12, 2}}, {"r12b", {X64_REG_12, 1}},
 
-        {"r13", {REG_13, 8}},  {"r13d", {REG_13, 4}},
-        {"r13w", {REG_13, 2}}, {"r13b", {REG_13, 1}},
+        {"r13", {X64_REG_13, 8}},  {"r13d", {X64_REG_13, 4}},
+        {"r13w", {X64_REG_13, 2}}, {"r13b", {X64_REG_13, 1}},
 
-        {"r14", {REG_14, 8}},  {"r14d", {REG_14, 4}},
-        {"r14w", {REG_14, 2}}, {"r14b", {REG_14, 1}},
+        {"r14", {X64_REG_14, 8}},  {"r14d", {X64_REG_14, 4}},
+        {"r14w", {X64_REG_14, 2}}, {"r14b", {X64_REG_14, 1}},
 
-        {"r15", {REG_15, 8}},  {"r15d", {REG_15, 4}},
-        {"r15w", {REG_15, 2}}, {"r15b", {REG_15, 1}},
+        {"r15", {X64_REG_15, 8}},  {"r15d", {X64_REG_15, 4}},
+        {"r15w", {X64_REG_15, 2}}, {"r15b", {X64_REG_15, 1}},
 
-        {"rip", {REG_RIP, 8}},
+        {"rip", {X64_REG_RIP, 8}},
+
+        {"xmm0", {X64_REG_XMM0, 16}},
+        {"xmm1", {X64_REG_XMM1, 16}},
+        {"xmm2", {X64_REG_XMM2, 16}},
+        {"xmm3", {X64_REG_XMM3, 16}},
+        {"xmm4", {X64_REG_XMM4, 16}},
+        {"xmm5", {X64_REG_XMM5, 16}},
+        {"xmm6", {X64_REG_XMM6, 16}},
+        {"xmm7", {X64_REG_XMM7, 16}},
+        {"xmm8", {X64_REG_XMM8, 16}},
+        {"xmm9", {X64_REG_XMM9, 16}},
+        {"xmm10", {X64_REG_XMM10, 16}},
+        {"xmm11", {X64_REG_XMM11, 16}},
+        {"xmm12", {X64_REG_XMM12, 16}},
+        {"xmm13", {X64_REG_XMM13, 16}},
+        {"xmm14", {X64_REG_XMM14, 16}},
+        {"xmm15", {X64_REG_XMM15, 16}},
 };
 
 void ArgumentParser_x64::reg_to_name(std::string *norm, Register reg) {
   switch (reg) {
-  case REG_A:
+  case X64_REG_A:
     *norm = "ax";
     break;
-  case REG_B:
+  case X64_REG_B:
     *norm = "bx";
     break;
-  case REG_C:
+  case X64_REG_C:
     *norm = "cx";
     break;
-  case REG_D:
+  case X64_REG_D:
     *norm = "dx";
     break;
 
-  case REG_SI:
+  case X64_REG_SI:
     *norm = "si";
     break;
-  case REG_DI:
+  case X64_REG_DI:
     *norm = "di";
     break;
-  case REG_BP:
+  case X64_REG_BP:
     *norm = "bp";
     break;
-  case REG_SP:
+  case X64_REG_SP:
     *norm = "sp";
     break;
 
-  case REG_8:
+  case X64_REG_8:
     *norm = "r8";
     break;
-  case REG_9:
+  case X64_REG_9:
     *norm = "r9";
     break;
-  case REG_10:
+  case X64_REG_10:
     *norm = "r10";
     break;
-  case REG_11:
+  case X64_REG_11:
     *norm = "r11";
     break;
-  case REG_12:
+  case X64_REG_12:
     *norm = "r12";
     break;
-  case REG_13:
+  case X64_REG_13:
     *norm = "r13";
     break;
-  case REG_14:
+  case X64_REG_14:
     *norm = "r14";
     break;
-  case REG_15:
+  case X64_REG_15:
     *norm = "r15";
     break;
 
-  case REG_RIP:
+  case X64_REG_RIP:
     *norm = "ip";
     break;
+
+  case X64_REG_XMM0:
+    *norm = "xmm0";
+    break;
+  case X64_REG_XMM1:
+    *norm = "xmm1";
+    break;
+  case X64_REG_XMM2:
+    *norm = "xmm2";
+    break;
+  case X64_REG_XMM3:
+    *norm = "xmm3";
+    break;
+  case X64_REG_XMM4:
+    *norm = "xmm4";
+    break;
+  case X64_REG_XMM5:
+    *norm = "xmm5";
+    break;
+  case X64_REG_XMM6:
+    *norm = "xmm6";
+    break;
+  case X64_REG_XMM7:
+    *norm = "xmm7";
+    break;
+  case X64_REG_XMM8:
+    *norm = "xmm8";
+    break;
+  case X64_REG_XMM9:
+    *norm = "xmm9";
+    break;
+  case X64_REG_XMM10:
+    *norm = "xmm10";
+    break;
+  case X64_REG_XMM11:
+    *norm = "xmm11";
+    break;
+  case X64_REG_XMM12:
+    *norm = "xmm12";
+    break;
+  case X64_REG_XMM13:
+    *norm = "xmm13";
+    break;
+  case X64_REG_XMM14:
+    *norm = "xmm14";
+    break;
+  case X64_REG_XMM15:
+    *norm = "xmm15";
+    break;
+
   }
 }
 
