@@ -179,6 +179,10 @@ class SmokeTests(TestCase):
     def test_ext4slower(self):
         self.run_with_int("ext4slower.py")
 
+    @skipUnless(kernel_version_ge(4,4), "requires kernel >= 4.4")
+    def test_f2fsslower(self):
+        self.run_with_int("f2fsslower.py", allow_early=True)
+
     @skipUnless(kernel_version_ge(4,4), "requires kernel >= 4.4")
     def test_filelife(self):
         self.run_with_int("filelife.py")
@@ -420,6 +424,10 @@ class SmokeTests(TestCase):
     def test_wakeuptime(self):
         self.run_with_duration("wakeuptime.py 1")
 
+    @skipUnless(kernel_version_ge(4,7), "requires kernel >= 4.7")
+    def test_wqlat(self):
+        self.run_with_int("wqlat.py 1 1", allow_early=True)
+
     def test_xfsdist(self):
         # Doesn't work on build bot because xfs functions not present in the
         # kernel image.
